# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .tenancy_type import TenancyType

__all__ = ["ClientUpdateParams"]


class ClientUpdateParams(TypedDict, total=False):
    can_auto_join_by_domain: Optional[bool]

    domain: Optional[str]

    logo_url: Optional[str]

    members_must_have_matching_domain: Optional[bool]

    name: Optional[str]

    tenancy_type: Optional[TenancyType]
    """Type of tenancy for a client."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel

__all__ = ["TenantAddResponse"]


class TenantAddResponse(BaseModel):
    """Response model for add tenants operation."""

    added: List[str]
    """Tenant IDs that were successfully added"""

    already_in_group: List[str]
    """Tenant IDs that were already members"""

    not_found: List[str]
    """Tenant IDs that don't exist"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from kater import Kater, AsyncKater
from tests.utils import assert_matches_type
from kater.types.v1 import (
    Connection,
    ConnectionListResponse,
    ConnectionSyncResponse,
    ConnectionRetrieveCredentialResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestConnections:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_overload_1(self, client: Kater) -> None:
        connection = client.v1.connections.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="postgresql",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params_overload_1(self, client: Kater) -> None:
        connection = client.v1.connections.create(
            databases=[
                {
                    "name": "x",
                    "description": "description",
                    "label": "label",
                    "schemas": [
                        {
                            "name": "x",
                            "description": "description",
                            "label": "label",
                        }
                    ],
                    "timezone": "UTC",
                }
            ],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="postgresql",
            database_timezone="database_timezone",
            description="description",
            label="label",
            port=1,
            query_timeout=1,
            query_timezone_conversion="query_timezone_conversion",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create_overload_1(self, client: Kater) -> None:
        response = client.v1.connections.with_raw_response.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="postgresql",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create_overload_1(self, client: Kater) -> None:
        with client.v1.connections.with_streaming_response.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="postgresql",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_overload_2(self, client: Kater) -> None:
        connection = client.v1.connections.create(
            account="account",
            auth={
                "auth_type": "password",
                "password": "password",
            },
            databases=[{"name": "x"}],
            name="name",
            role="role",
            username="username",
            warehouse="warehouse",
            warehouse_type="snowflake",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params_overload_2(self, client: Kater) -> None:
        connection = client.v1.connections.create(
            account="account",
            auth={
                "auth_type": "password",
                "password": "password",
            },
            databases=[
                {
                    "name": "x",
                    "description": "description",
                    "label": "label",
                    "schemas": [
                        {
                            "name": "x",
                            "description": "description",
                            "label": "label",
                        }
                    ],
                    "timezone": "UTC",
                }
            ],
            name="name",
            role="role",
            username="username",
            warehouse="warehouse",
            warehouse_type="snowflake",
            database_timezone="database_timezone",
            description="description",
            label="label",
            query_timeout=1,
            query_timezone_conversion="query_timezone_conversion",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create_overload_2(self, client: Kater) -> None:
        response = client.v1.connections.with_raw_response.create(
            account="account",
            auth={
                "auth_type": "password",
                "password": "password",
            },
            databases=[{"name": "x"}],
            name="name",
            role="role",
            username="username",
            warehouse="warehouse",
            warehouse_type="snowflake",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create_overload_2(self, client: Kater) -> None:
        with client.v1.connections.with_streaming_response.create(
            account="account",
            auth={
                "auth_type": "password",
                "password": "password",
            },
            databases=[{"name": "x"}],
            name="name",
            role="role",
            username="username",
            warehouse="warehouse",
            warehouse_type="snowflake",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_overload_3(self, client: Kater) -> None:
        connection = client.v1.connections.create(
            access_token="access_token",
            databases=[{"name": "x"}],
            http_path="http_path",
            name="name",
            server_hostname="server_hostname",
            warehouse_type="databricks",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params_overload_3(self, client: Kater) -> None:
        connection = client.v1.connections.create(
            access_token="access_token",
            databases=[
                {
                    "name": "x",
                    "description": "description",
                    "label": "label",
                    "schemas": [
                        {
                            "name": "x",
                            "description": "description",
                            "label": "label",
                        }
                    ],
                    "timezone": "UTC",
                }
            ],
            http_path="http_path",
            name="name",
            server_hostname="server_hostname",
            warehouse_type="databricks",
            database_timezone="database_timezone",
            description="description",
            label="label",
            query_timeout=1,
            query_timezone_conversion="query_timezone_conversion",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create_overload_3(self, client: Kater) -> None:
        response = client.v1.connections.with_raw_response.create(
            access_token="access_token",
            databases=[{"name": "x"}],
            http_path="http_path",
            name="name",
            server_hostname="server_hostname",
            warehouse_type="databricks",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create_overload_3(self, client: Kater) -> None:
        with client.v1.connections.with_streaming_response.create(
            access_token="access_token",
            databases=[{"name": "x"}],
            http_path="http_path",
            name="name",
            server_hostname="server_hostname",
            warehouse_type="databricks",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_overload_4(self, client: Kater) -> None:
        connection = client.v1.connections.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="clickhouse",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params_overload_4(self, client: Kater) -> None:
        connection = client.v1.connections.create(
            databases=[
                {
                    "name": "x",
                    "description": "description",
                    "label": "label",
                    "schemas": [
                        {
                            "name": "x",
                            "description": "description",
                            "label": "label",
                        }
                    ],
                    "timezone": "UTC",
                }
            ],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="clickhouse",
            database_timezone="database_timezone",
            description="description",
            label="label",
            port=1,
            query_timeout=1,
            query_timezone_conversion="query_timezone_conversion",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create_overload_4(self, client: Kater) -> None:
        response = client.v1.connections.with_raw_response.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="clickhouse",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create_overload_4(self, client: Kater) -> None:
        with client.v1.connections.with_streaming_response.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="clickhouse",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_overload_5(self, client: Kater) -> None:
        connection = client.v1.connections.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="mssql",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params_overload_5(self, client: Kater) -> None:
        connection = client.v1.connections.create(
            databases=[
                {
                    "name": "x",
                    "description": "description",
                    "label": "label",
                    "schemas": [
                        {
                            "name": "x",
                            "description": "description",
                            "label": "label",
                        }
                    ],
                    "timezone": "UTC",
                }
            ],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="mssql",
            database_timezone="database_timezone",
            description="description",
            label="label",
            port=1,
            query_timeout=1,
            query_timezone_conversion="query_timezone_conversion",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create_overload_5(self, client: Kater) -> None:
        response = client.v1.connections.with_raw_response.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="mssql",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create_overload_5(self, client: Kater) -> None:
        with client.v1.connections.with_streaming_response.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="mssql",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Kater) -> None:
        connection = client.v1.connections.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Kater) -> None:
        response = client.v1.connections.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Kater) -> None:
        with client.v1.connections.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Kater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            client.v1.connections.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Kater) -> None:
        connection = client.v1.connections.update(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Kater) -> None:
        connection = client.v1.connections.update(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            label="label",
            name="name",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Kater) -> None:
        response = client.v1.connections.with_raw_response.update(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Kater) -> None:
        with client.v1.connections.with_streaming_response.update(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Kater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            client.v1.connections.with_raw_response.update(
                connection_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Kater) -> None:
        connection = client.v1.connections.list()
        assert_matches_type(ConnectionListResponse, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Kater) -> None:
        response = client.v1.connections.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = response.parse()
        assert_matches_type(ConnectionListResponse, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Kater) -> None:
        with client.v1.connections.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = response.parse()
            assert_matches_type(ConnectionListResponse, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Kater) -> None:
        connection = client.v1.connections.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert connection is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Kater) -> None:
        response = client.v1.connections.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = response.parse()
        assert connection is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Kater) -> None:
        with client.v1.connections.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = response.parse()
            assert connection is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Kater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            client.v1.connections.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_credential(self, client: Kater) -> None:
        connection = client.v1.connections.retrieve_credential(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(ConnectionRetrieveCredentialResponse, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_credential(self, client: Kater) -> None:
        response = client.v1.connections.with_raw_response.retrieve_credential(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = response.parse()
        assert_matches_type(ConnectionRetrieveCredentialResponse, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_credential(self, client: Kater) -> None:
        with client.v1.connections.with_streaming_response.retrieve_credential(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = response.parse()
            assert_matches_type(ConnectionRetrieveCredentialResponse, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve_credential(self, client: Kater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            client.v1.connections.with_raw_response.retrieve_credential(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_sync(self, client: Kater) -> None:
        connection = client.v1.connections.sync(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(ConnectionSyncResponse, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_sync(self, client: Kater) -> None:
        response = client.v1.connections.with_raw_response.sync(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = response.parse()
        assert_matches_type(ConnectionSyncResponse, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_sync(self, client: Kater) -> None:
        with client.v1.connections.with_streaming_response.sync(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = response.parse()
            assert_matches_type(ConnectionSyncResponse, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_sync(self, client: Kater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            client.v1.connections.with_raw_response.sync(
                "",
            )


class TestAsyncConnections:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_overload_1(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="postgresql",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params_overload_1(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.create(
            databases=[
                {
                    "name": "x",
                    "description": "description",
                    "label": "label",
                    "schemas": [
                        {
                            "name": "x",
                            "description": "description",
                            "label": "label",
                        }
                    ],
                    "timezone": "UTC",
                }
            ],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="postgresql",
            database_timezone="database_timezone",
            description="description",
            label="label",
            port=1,
            query_timeout=1,
            query_timezone_conversion="query_timezone_conversion",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create_overload_1(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.with_raw_response.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="postgresql",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = await response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create_overload_1(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.with_streaming_response.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="postgresql",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = await response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_overload_2(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.create(
            account="account",
            auth={
                "auth_type": "password",
                "password": "password",
            },
            databases=[{"name": "x"}],
            name="name",
            role="role",
            username="username",
            warehouse="warehouse",
            warehouse_type="snowflake",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params_overload_2(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.create(
            account="account",
            auth={
                "auth_type": "password",
                "password": "password",
            },
            databases=[
                {
                    "name": "x",
                    "description": "description",
                    "label": "label",
                    "schemas": [
                        {
                            "name": "x",
                            "description": "description",
                            "label": "label",
                        }
                    ],
                    "timezone": "UTC",
                }
            ],
            name="name",
            role="role",
            username="username",
            warehouse="warehouse",
            warehouse_type="snowflake",
            database_timezone="database_timezone",
            description="description",
            label="label",
            query_timeout=1,
            query_timezone_conversion="query_timezone_conversion",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create_overload_2(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.with_raw_response.create(
            account="account",
            auth={
                "auth_type": "password",
                "password": "password",
            },
            databases=[{"name": "x"}],
            name="name",
            role="role",
            username="username",
            warehouse="warehouse",
            warehouse_type="snowflake",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = await response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create_overload_2(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.with_streaming_response.create(
            account="account",
            auth={
                "auth_type": "password",
                "password": "password",
            },
            databases=[{"name": "x"}],
            name="name",
            role="role",
            username="username",
            warehouse="warehouse",
            warehouse_type="snowflake",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = await response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_overload_3(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.create(
            access_token="access_token",
            databases=[{"name": "x"}],
            http_path="http_path",
            name="name",
            server_hostname="server_hostname",
            warehouse_type="databricks",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params_overload_3(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.create(
            access_token="access_token",
            databases=[
                {
                    "name": "x",
                    "description": "description",
                    "label": "label",
                    "schemas": [
                        {
                            "name": "x",
                            "description": "description",
                            "label": "label",
                        }
                    ],
                    "timezone": "UTC",
                }
            ],
            http_path="http_path",
            name="name",
            server_hostname="server_hostname",
            warehouse_type="databricks",
            database_timezone="database_timezone",
            description="description",
            label="label",
            query_timeout=1,
            query_timezone_conversion="query_timezone_conversion",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create_overload_3(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.with_raw_response.create(
            access_token="access_token",
            databases=[{"name": "x"}],
            http_path="http_path",
            name="name",
            server_hostname="server_hostname",
            warehouse_type="databricks",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = await response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create_overload_3(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.with_streaming_response.create(
            access_token="access_token",
            databases=[{"name": "x"}],
            http_path="http_path",
            name="name",
            server_hostname="server_hostname",
            warehouse_type="databricks",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = await response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_overload_4(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="clickhouse",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params_overload_4(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.create(
            databases=[
                {
                    "name": "x",
                    "description": "description",
                    "label": "label",
                    "schemas": [
                        {
                            "name": "x",
                            "description": "description",
                            "label": "label",
                        }
                    ],
                    "timezone": "UTC",
                }
            ],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="clickhouse",
            database_timezone="database_timezone",
            description="description",
            label="label",
            port=1,
            query_timeout=1,
            query_timezone_conversion="query_timezone_conversion",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create_overload_4(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.with_raw_response.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="clickhouse",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = await response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create_overload_4(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.with_streaming_response.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="clickhouse",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = await response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_overload_5(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="mssql",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params_overload_5(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.create(
            databases=[
                {
                    "name": "x",
                    "description": "description",
                    "label": "label",
                    "schemas": [
                        {
                            "name": "x",
                            "description": "description",
                            "label": "label",
                        }
                    ],
                    "timezone": "UTC",
                }
            ],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="mssql",
            database_timezone="database_timezone",
            description="description",
            label="label",
            port=1,
            query_timeout=1,
            query_timezone_conversion="query_timezone_conversion",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create_overload_5(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.with_raw_response.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="mssql",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = await response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create_overload_5(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.with_streaming_response.create(
            databases=[{"name": "x"}],
            host="host",
            name="name",
            password="password",
            username="username",
            warehouse_type="mssql",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = await response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = await response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = await response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncKater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            await async_client.v1.connections.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.update(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.update(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            label="label",
            name="name",
        )
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.with_raw_response.update(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = await response.parse()
        assert_matches_type(Connection, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.with_streaming_response.update(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = await response.parse()
            assert_matches_type(Connection, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncKater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            await async_client.v1.connections.with_raw_response.update(
                connection_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.list()
        assert_matches_type(ConnectionListResponse, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = await response.parse()
        assert_matches_type(ConnectionListResponse, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = await response.parse()
            assert_matches_type(ConnectionListResponse, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert connection is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = await response.parse()
        assert connection is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = await response.parse()
            assert connection is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncKater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            await async_client.v1.connections.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_credential(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.retrieve_credential(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(ConnectionRetrieveCredentialResponse, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_credential(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.with_raw_response.retrieve_credential(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = await response.parse()
        assert_matches_type(ConnectionRetrieveCredentialResponse, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_credential(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.with_streaming_response.retrieve_credential(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = await response.parse()
            assert_matches_type(ConnectionRetrieveCredentialResponse, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve_credential(self, async_client: AsyncKater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            await async_client.v1.connections.with_raw_response.retrieve_credential(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_sync(self, async_client: AsyncKater) -> None:
        connection = await async_client.v1.connections.sync(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(ConnectionSyncResponse, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_sync(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.with_raw_response.sync(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection = await response.parse()
        assert_matches_type(ConnectionSyncResponse, connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_sync(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.with_streaming_response.sync(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection = await response.parse()
            assert_matches_type(ConnectionSyncResponse, connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_sync(self, async_client: AsyncKater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            await async_client.v1.connections.with_raw_response.sync(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ConnectionSyncResponse"]


class ConnectionSyncResponse(BaseModel):
    """Response for syncing views.

    Returned after successfully creating a PR with merged ViewSchema files,
    or indicating that all views are already up to date.
    """

    views_updated: int
    """Number of views in the PR"""

    message: Optional[str] = None
    """Status message"""

    pr_number: Optional[int] = None
    """GitHub PR number"""

    pr_url: Optional[str] = None
    """GitHub PR URL"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from .repos import (
    ReposResource,
    AsyncReposResource,
    ReposResourceWithRawResponse,
    AsyncReposResourceWithRawResponse,
    ReposResourceWithStreamingResponse,
    AsyncReposResourceWithStreamingResponse,
)
from .scaffold import (
    ScaffoldResource,
    AsyncScaffoldResource,
    ScaffoldResourceWithRawResponse,
    AsyncScaffoldResourceWithRawResponse,
    ScaffoldResourceWithStreamingResponse,
    AsyncScaffoldResourceWithStreamingResponse,
)
from .webhooks import (
    WebhooksResource,
    AsyncWebhooksResource,
    WebhooksResourceWithRawResponse,
    AsyncWebhooksResourceWithRawResponse,
    WebhooksResourceWithStreamingResponse,
    AsyncWebhooksResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ....types.v1 import github_connect_params, github_callback_params
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.github_connect_response import GitHubConnectResponse
from ....types.v1.github_get_status_response import GitHubGetStatusResponse
from ....types.v1.github_get_installation_link_response import GitHubGetInstallationLinkResponse

__all__ = ["GitHubResource", "AsyncGitHubResource"]


class GitHubResource(SyncAPIResource):
    @cached_property
    def repos(self) -> ReposResource:
        return ReposResource(self._client)

    @cached_property
    def scaffold(self) -> ScaffoldResource:
        return ScaffoldResource(self._client)

    @cached_property
    def webhooks(self) -> WebhooksResource:
        return WebhooksResource(self._client)

    @cached_property
    def with_raw_response(self) -> GitHubResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return GitHubResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GitHubResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return GitHubResourceWithStreamingResponse(self)

    def callback(
        self,
        *,
        state: str,
        code: Optional[str] | Omit = omit,
        error: Optional[str] | Omit = omit,
        error_description: Optional[str] | Omit = omit,
        installation_id: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Handles the OAuth callback from GitHub after user authorization.

        Args:
          state: State token for CSRF validation

          code: OAuth authorization code

          error: OAuth error code from GitHub

          error_description: OAuth error description

          installation_id: GitHub App installation ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/api/v1/github/callback",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "state": state,
                        "code": code,
                        "error": error,
                        "error_description": error_description,
                        "installation_id": installation_id,
                    },
                    github_callback_params.GitHubCallbackParams,
                ),
            ),
            cast_to=object,
        )

    def connect(
        self,
        *,
        return_to: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GitHubConnectResponse:
        """
        Initiates the GitHub App installation flow.

        Args:
          return_to: Frontend path to redirect to after OAuth success (e.g., '/' or
              '/settings/github')

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/api/v1/github/connect",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"return_to": return_to}, github_connect_params.GitHubConnectParams),
            ),
            cast_to=GitHubConnectResponse,
        )

    def get_installation_link(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GitHubGetInstallationLinkResponse:
        """Generates a shareable GitHub App installation link."""
        return self._get(
            "/api/v1/github/installation-link",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GitHubGetInstallationLinkResponse,
        )

    def get_status(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GitHubGetStatusResponse:
        """Returns the current GitHub connection status for the client."""
        return self._get(
            "/api/v1/github/status",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GitHubGetStatusResponse,
        )


class AsyncGitHubResource(AsyncAPIResource):
    @cached_property
    def repos(self) -> AsyncReposResource:
        return AsyncReposResource(self._client)

    @cached_property
    def scaffold(self) -> AsyncScaffoldResource:
        return AsyncScaffoldResource(self._client)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResource:
        return AsyncWebhooksResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncGitHubResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncGitHubResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGitHubResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncGitHubResourceWithStreamingResponse(self)

    async def callback(
        self,
        *,
        state: str,
        code: Optional[str] | Omit = omit,
        error: Optional[str] | Omit = omit,
        error_description: Optional[str] | Omit = omit,
        installation_id: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Handles the OAuth callback from GitHub after user authorization.

        Args:
          state: State token for CSRF validation

          code: OAuth authorization code

          error: OAuth error code from GitHub

          error_description: OAuth error description

          installation_id: GitHub App installation ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/api/v1/github/callback",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "state": state,
                        "code": code,
                        "error": error,
                        "error_description": error_description,
                        "installation_id": installation_id,
                    },
                    github_callback_params.GitHubCallbackParams,
                ),
            ),
            cast_to=object,
        )

    async def connect(
        self,
        *,
        return_to: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GitHubConnectResponse:
        """
        Initiates the GitHub App installation flow.

        Args:
          return_to: Frontend path to redirect to after OAuth success (e.g., '/' or
              '/settings/github')

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/api/v1/github/connect",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"return_to": return_to}, github_connect_params.GitHubConnectParams),
            ),
            cast_to=GitHubConnectResponse,
        )

    async def get_installation_link(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GitHubGetInstallationLinkResponse:
        """Generates a shareable GitHub App installation link."""
        return await self._get(
            "/api/v1/github/installation-link",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GitHubGetInstallationLinkResponse,
        )

    async def get_status(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GitHubGetStatusResponse:
        """Returns the current GitHub connection status for the client."""
        return await self._get(
            "/api/v1/github/status",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GitHubGetStatusResponse,
        )


class GitHubResourceWithRawResponse:
    def __init__(self, github: GitHubResource) -> None:
        self._github = github

        self.callback = to_raw_response_wrapper(
            github.callback,
        )
        self.connect = to_raw_response_wrapper(
            github.connect,
        )
        self.get_installation_link = to_raw_response_wrapper(
            github.get_installation_link,
        )
        self.get_status = to_raw_response_wrapper(
            github.get_status,
        )

    @cached_property
    def repos(self) -> ReposResourceWithRawResponse:
        return ReposResourceWithRawResponse(self._github.repos)

    @cached_property
    def scaffold(self) -> ScaffoldResourceWithRawResponse:
        return ScaffoldResourceWithRawResponse(self._github.scaffold)

    @cached_property
    def webhooks(self) -> WebhooksResourceWithRawResponse:
        return WebhooksResourceWithRawResponse(self._github.webhooks)


class AsyncGitHubResourceWithRawResponse:
    def __init__(self, github: AsyncGitHubResource) -> None:
        self._github = github

        self.callback = async_to_raw_response_wrapper(
            github.callback,
        )
        self.connect = async_to_raw_response_wrapper(
            github.connect,
        )
        self.get_installation_link = async_to_raw_response_wrapper(
            github.get_installation_link,
        )
        self.get_status = async_to_raw_response_wrapper(
            github.get_status,
        )

    @cached_property
    def repos(self) -> AsyncReposResourceWithRawResponse:
        return AsyncReposResourceWithRawResponse(self._github.repos)

    @cached_property
    def scaffold(self) -> AsyncScaffoldResourceWithRawResponse:
        return AsyncScaffoldResourceWithRawResponse(self._github.scaffold)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResourceWithRawResponse:
        return AsyncWebhooksResourceWithRawResponse(self._github.webhooks)


class GitHubResourceWithStreamingResponse:
    def __init__(self, github: GitHubResource) -> None:
        self._github = github

        self.callback = to_streamed_response_wrapper(
            github.callback,
        )
        self.connect = to_streamed_response_wrapper(
            github.connect,
        )
        self.get_installation_link = to_streamed_response_wrapper(
            github.get_installation_link,
        )
        self.get_status = to_streamed_response_wrapper(
            github.get_status,
        )

    @cached_property
    def repos(self) -> ReposResourceWithStreamingResponse:
        return ReposResourceWithStreamingResponse(self._github.repos)

    @cached_property
    def scaffold(self) -> ScaffoldResourceWithStreamingResponse:
        return ScaffoldResourceWithStreamingResponse(self._github.scaffold)

    @cached_property
    def webhooks(self) -> WebhooksResourceWithStreamingResponse:
        return WebhooksResourceWithStreamingResponse(self._github.webhooks)


class AsyncGitHubResourceWithStreamingResponse:
    def __init__(self, github: AsyncGitHubResource) -> None:
        self._github = github

        self.callback = async_to_streamed_response_wrapper(
            github.callback,
        )
        self.connect = async_to_streamed_response_wrapper(
            github.connect,
        )
        self.get_installation_link = async_to_streamed_response_wrapper(
            github.get_installation_link,
        )
        self.get_status = async_to_streamed_response_wrapper(
            github.get_status,
        )

    @cached_property
    def repos(self) -> AsyncReposResourceWithStreamingResponse:
        return AsyncReposResourceWithStreamingResponse(self._github.repos)

    @cached_property
    def scaffold(self) -> AsyncScaffoldResourceWithStreamingResponse:
        return AsyncScaffoldResourceWithStreamingResponse(self._github.scaffold)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResourceWithStreamingResponse:
        return AsyncWebhooksResourceWithStreamingResponse(self._github.webhooks)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["GitHubGetStatusResponse"]


class GitHubGetStatusResponse(BaseModel):
    """Response for connection status."""

    connected: bool

    default_branch: Optional[str] = None

    last_sync_at: Optional[str] = None

    last_used_at: Optional[str] = None

    owner: Optional[str] = None

    repository: Optional[str] = None

    scaffolding_pr_url: Optional[str] = None

    scaffolding_status: Optional[str] = None

    status: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .import_tenants import ImportTenants as ImportTenants
from .batch_tenant_error import BatchTenantError as BatchTenantError
from .batch_create_params import BatchCreateParams as BatchCreateParams
from .batch_delete_params import BatchDeleteParams as BatchDeleteParams
from .batch_update_params import BatchUpdateParams as BatchUpdateParams
from .batch_tenant_success import BatchTenantSuccess as BatchTenantSuccess
from .batch_create_response import BatchCreateResponse as BatchCreateResponse
from .batch_delete_response import BatchDeleteResponse as BatchDeleteResponse
from .batch_update_response import BatchUpdateResponse as BatchUpdateResponse
from .import_from_csv_params import ImportFromCsvParams as ImportFromCsvParams
from .import_from_warehouse_params import ImportFromWarehouseParams as ImportFromWarehouseParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["TenantUpdateParams"]


class TenantUpdateParams(TypedDict, total=False):
    connection_id: Optional[str]
    """Connection ID for database association (required with database_name)"""

    database_name: Optional[str]
    """Database name for association (required with connection_id)"""

    group_names: Optional[SequenceNotStr[str]]
    """New list of group names (replaces all existing associations)"""

    tenant_name: Optional[str]
    """New human-readable tenant name"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.github.scaffold_trigger import ScaffoldTrigger

__all__ = ["ScaffoldResource", "AsyncScaffoldResource"]


class ScaffoldResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ScaffoldResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ScaffoldResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ScaffoldResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return ScaffoldResourceWithStreamingResponse(self)

    def retry(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScaffoldTrigger:
        """Clean up partial state and retry scaffolding from scratch."""
        return self._post(
            "/api/v1/github/scaffold/retry",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScaffoldTrigger,
        )

    def trigger(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScaffoldTrigger:
        """Create the Kater folder structure and PR in the selected repository."""
        return self._post(
            "/api/v1/github/scaffold",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScaffoldTrigger,
        )


class AsyncScaffoldResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncScaffoldResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncScaffoldResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncScaffoldResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncScaffoldResourceWithStreamingResponse(self)

    async def retry(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScaffoldTrigger:
        """Clean up partial state and retry scaffolding from scratch."""
        return await self._post(
            "/api/v1/github/scaffold/retry",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScaffoldTrigger,
        )

    async def trigger(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScaffoldTrigger:
        """Create the Kater folder structure and PR in the selected repository."""
        return await self._post(
            "/api/v1/github/scaffold",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScaffoldTrigger,
        )


class ScaffoldResourceWithRawResponse:
    def __init__(self, scaffold: ScaffoldResource) -> None:
        self._scaffold = scaffold

        self.retry = to_raw_response_wrapper(
            scaffold.retry,
        )
        self.trigger = to_raw_response_wrapper(
            scaffold.trigger,
        )


class AsyncScaffoldResourceWithRawResponse:
    def __init__(self, scaffold: AsyncScaffoldResource) -> None:
        self._scaffold = scaffold

        self.retry = async_to_raw_response_wrapper(
            scaffold.retry,
        )
        self.trigger = async_to_raw_response_wrapper(
            scaffold.trigger,
        )


class ScaffoldResourceWithStreamingResponse:
    def __init__(self, scaffold: ScaffoldResource) -> None:
        self._scaffold = scaffold

        self.retry = to_streamed_response_wrapper(
            scaffold.retry,
        )
        self.trigger = to_streamed_response_wrapper(
            scaffold.trigger,
        )


class AsyncScaffoldResourceWithStreamingResponse:
    def __init__(self, scaffold: AsyncScaffoldResource) -> None:
        self._scaffold = scaffold

        self.retry = async_to_streamed_response_wrapper(
            scaffold.retry,
        )
        self.trigger = async_to_streamed_response_wrapper(
            scaffold.trigger,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

__all__ = ["DatabaseConfigParam", "Schema"]


class Schema(TypedDict, total=False):
    """Schema configuration for connection creation request."""

    name: Required[str]
    """Schema name"""

    description: Optional[str]
    """Description of the schema"""

    label: Optional[str]
    """Human-readable label for the schema (defaults to name if not set)"""


class DatabaseConfigParam(TypedDict, total=False):
    """Database configuration for connection creation request."""

    name: Required[str]
    """Database name"""

    description: Optional[str]
    """Description of the database"""

    label: Optional[str]
    """Human-readable label for the database (defaults to name if not set)"""

    schemas: Iterable[Schema]
    """Schema configurations to include (empty = discover all schemas)"""

    timezone: Optional[str]
    """Timezone for the database (e.g., 'UTC', 'America/New_York')"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import TypeAlias

from ..._models import BaseModel

__all__ = ["GroupListResponse", "GroupListResponseItem"]


class GroupListResponseItem(BaseModel):
    """Response model for a group in list responses (with tenant count)."""

    id: str
    """Group ID"""

    created_at: datetime
    """Creation timestamp"""

    description: Optional[str] = None
    """Group description"""

    name: str
    """Group name"""

    tenant_count: int
    """Number of tenants in this group"""

    updated_at: datetime
    """Last update timestamp"""


GroupListResponse: TypeAlias = List[GroupListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["BatchUpdateParams", "Tenant"]


class BatchUpdateParams(TypedDict, total=False):
    tenants: Required[Iterable[Tenant]]
    """List of tenant updates"""


class Tenant(TypedDict, total=False):
    """Single tenant update in a batch operation."""

    tenant_id: Required[str]
    """Tenant ID to update"""

    connection_id: Optional[str]
    """Connection ID for database association (required with database_name)"""

    database_name: Optional[str]
    """Database name for association (required with connection_id)"""

    group_names: Optional[SequenceNotStr[str]]
    """New list of group names (replaces all existing associations)"""

    tenant_name: Optional[str]
    """New human-readable tenant name"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["GitHubConnectParams"]


class GitHubConnectParams(TypedDict, total=False):
    return_to: Optional[str]
    """
    Frontend path to redirect to after OAuth success (e.g., '/' or
    '/settings/github')
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .database_config_param import DatabaseConfigParam

__all__ = [
    "ConnectionCreateParams",
    "PostgresConnectionConfig",
    "SnowflakeConnectionConfig",
    "SnowflakeConnectionConfigAuth",
    "SnowflakeConnectionConfigAuthSnowflakePasswordAuth",
    "SnowflakeConnectionConfigAuthSnowflakePrivateKeyAuth",
    "DatabricksConnectionConfig",
    "ClickHouseConnectionConfig",
    "MssqlConnectionConfig",
]


class PostgresConnectionConfig(TypedDict, total=False):
    databases: Required[Iterable[DatabaseConfigParam]]
    """Databases to include in the connection (at least one required)"""

    host: Required[str]
    """Database host"""

    name: Required[str]
    """Name of the connection (snake_case: lowercase letters, numbers, underscores)"""

    password: Required[str]
    """Database password"""

    username: Required[str]
    """Database username"""

    warehouse_type: Required[Literal["postgresql"]]
    """Warehouse type"""

    database_timezone: Optional[str]
    """Default timezone for the connection (e.g., 'UTC', 'America/New_York')"""

    description: Optional[str]
    """Description of the connection"""

    label: Optional[str]
    """Human-readable label for the connection (defaults to name if not set)"""

    port: int
    """Database port"""

    query_timeout: Optional[int]
    """Query timeout in seconds (1-3600)"""

    query_timezone_conversion: Optional[str]
    """Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'"""


class SnowflakeConnectionConfig(TypedDict, total=False):
    account: Required[str]
    """Snowflake account identifier (e.g., 'xy12345.us-east-1')"""

    auth: Required[SnowflakeConnectionConfigAuth]
    """Authentication configuration"""

    databases: Required[Iterable[DatabaseConfigParam]]
    """Databases to include in the connection (at least one required)"""

    name: Required[str]
    """Name of the connection (snake_case: lowercase letters, numbers, underscores)"""

    role: Required[str]
    """Snowflake role"""

    username: Required[str]
    """Snowflake username"""

    warehouse: Required[str]
    """Compute warehouse name"""

    warehouse_type: Required[Literal["snowflake"]]
    """Warehouse type"""

    database_timezone: Optional[str]
    """Default timezone for the connection (e.g., 'UTC', 'America/New_York')"""

    description: Optional[str]
    """Description of the connection"""

    label: Optional[str]
    """Human-readable label for the connection (defaults to name if not set)"""

    query_timeout: Optional[int]
    """Query timeout in seconds (1-3600)"""

    query_timezone_conversion: Optional[str]
    """Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'"""


class SnowflakeConnectionConfigAuthSnowflakePasswordAuth(TypedDict, total=False):
    """Snowflake password authentication."""

    auth_type: Required[Literal["password"]]
    """Authentication type"""

    password: Required[str]
    """Snowflake password"""


class SnowflakeConnectionConfigAuthSnowflakePrivateKeyAuth(TypedDict, total=False):
    """Snowflake private key authentication."""

    auth_type: Required[Literal["private_key"]]
    """Authentication type"""

    private_key: Required[str]
    """PEM-encoded private key"""

    private_key_passphrase: Optional[str]
    """Passphrase if key is encrypted"""


SnowflakeConnectionConfigAuth: TypeAlias = Union[
    SnowflakeConnectionConfigAuthSnowflakePasswordAuth, SnowflakeConnectionConfigAuthSnowflakePrivateKeyAuth
]


class DatabricksConnectionConfig(TypedDict, total=False):
    access_token: Required[str]
    """Databricks personal access token"""

    databases: Required[Iterable[DatabaseConfigParam]]
    """Databases to include in the connection (at least one required)"""

    http_path: Required[str]
    """SQL warehouse HTTP path (e.g., '/sql/1.0/warehouses/xxx')"""

    name: Required[str]
    """Name of the connection (snake_case: lowercase letters, numbers, underscores)"""

    server_hostname: Required[str]
    """Databricks server hostname (e.g., 'dbc-xxx.cloud.databricks.com')"""

    warehouse_type: Required[Literal["databricks"]]
    """Warehouse type"""

    database_timezone: Optional[str]
    """Default timezone for the connection (e.g., 'UTC', 'America/New_York')"""

    description: Optional[str]
    """Description of the connection"""

    label: Optional[str]
    """Human-readable label for the connection (defaults to name if not set)"""

    query_timeout: Optional[int]
    """Query timeout in seconds (1-3600)"""

    query_timezone_conversion: Optional[str]
    """Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'"""


class ClickHouseConnectionConfig(TypedDict, total=False):
    databases: Required[Iterable[DatabaseConfigParam]]
    """Databases to include in the connection (at least one required)"""

    host: Required[str]
    """ClickHouse host"""

    name: Required[str]
    """Name of the connection (snake_case: lowercase letters, numbers, underscores)"""

    password: Required[str]
    """ClickHouse password"""

    username: Required[str]
    """ClickHouse username"""

    warehouse_type: Required[Literal["clickhouse"]]
    """Warehouse type"""

    database_timezone: Optional[str]
    """Default timezone for the connection (e.g., 'UTC', 'America/New_York')"""

    description: Optional[str]
    """Description of the connection"""

    label: Optional[str]
    """Human-readable label for the connection (defaults to name if not set)"""

    port: int
    """ClickHouse port"""

    query_timeout: Optional[int]
    """Query timeout in seconds (1-3600)"""

    query_timezone_conversion: Optional[str]
    """Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'"""


class MssqlConnectionConfig(TypedDict, total=False):
    databases: Required[Iterable[DatabaseConfigParam]]
    """Databases to include in the connection (at least one required)"""

    host: Required[str]
    """SQL Server host"""

    name: Required[str]
    """Name of the connection (snake_case: lowercase letters, numbers, underscores)"""

    password: Required[str]
    """SQL Server password"""

    username: Required[str]
    """SQL Server username"""

    warehouse_type: Required[Literal["mssql"]]
    """Warehouse type"""

    database_timezone: Optional[str]
    """Default timezone for the connection (e.g., 'UTC', 'America/New_York')"""

    description: Optional[str]
    """Description of the connection"""

    label: Optional[str]
    """Human-readable label for the connection (defaults to name if not set)"""

    port: int
    """SQL Server port"""

    query_timeout: Optional[int]
    """Query timeout in seconds (1-3600)"""

    query_timezone_conversion: Optional[str]
    """Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'"""


ConnectionCreateParams: TypeAlias = Union[
    PostgresConnectionConfig,
    SnowflakeConnectionConfig,
    DatabricksConnectionConfig,
    ClickHouseConnectionConfig,
    MssqlConnectionConfig,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..tenant import Tenant
from ...._models import BaseModel

__all__ = ["BatchTenantSuccess"]


class BatchTenantSuccess(BaseModel):
    """Successful tenant operation in batch response."""

    tenant_id: str
    """Tenant ID"""

    tenant: Optional[Tenant] = None
    """Response model for a single tenant."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["UserDeleteResponse"]


class UserDeleteResponse(BaseModel):
    """Response schema for delete operations."""

    message: str

    success: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel
from ..connection import Connection

__all__ = ["DatabaseUpdateResponse"]


class DatabaseUpdateResponse(BaseModel):
    """Response for database/schema updates that may involve a rename PR."""

    connection: Connection
    """Updated connection"""

    pr_number: Optional[int] = None
    """GitHub PR number"""

    pr_status: Optional[str] = None
    """PR status: 'open' or 'merged'"""

    pr_url: Optional[str] = None
    """GitHub PR URL (if rename PR created)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tenant_add_params import TenantAddParams as TenantAddParams
from .tenant_add_response import TenantAddResponse as TenantAddResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .tenant import Tenant

__all__ = ["TenantListResponse"]

TenantListResponse: TypeAlias = List[Tenant]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["GitHubGetInstallationLinkResponse"]


class GitHubGetInstallationLinkResponse(BaseModel):
    """Response for installation link generation."""

    app_name: str

    installation_url: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .databases import (
    DatabasesResource,
    AsyncDatabasesResource,
    DatabasesResourceWithRawResponse,
    AsyncDatabasesResourceWithRawResponse,
    DatabasesResourceWithStreamingResponse,
    AsyncDatabasesResourceWithStreamingResponse,
)
from .connections import (
    ConnectionsResource,
    AsyncConnectionsResource,
    ConnectionsResourceWithRawResponse,
    AsyncConnectionsResourceWithRawResponse,
    ConnectionsResourceWithStreamingResponse,
    AsyncConnectionsResourceWithStreamingResponse,
)

__all__ = [
    "DatabasesResource",
    "AsyncDatabasesResource",
    "DatabasesResourceWithRawResponse",
    "AsyncDatabasesResourceWithRawResponse",
    "DatabasesResourceWithStreamingResponse",
    "AsyncDatabasesResourceWithStreamingResponse",
    "ConnectionsResource",
    "AsyncConnectionsResource",
    "ConnectionsResourceWithRawResponse",
    "AsyncConnectionsResourceWithRawResponse",
    "ConnectionsResourceWithStreamingResponse",
    "AsyncConnectionsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["ScaffoldTrigger"]


class ScaffoldTrigger(BaseModel):
    """Response from triggering scaffolding."""

    message: str

    status: str

    success: bool

    branch: Optional[str] = None

    error: Optional[str] = None

    pr_number: Optional[int] = None

    pr_url: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["Repository"]


class Repository(BaseModel):
    """Repository information response."""

    id: int

    default_branch: str

    full_name: str

    has_kater_structure: bool

    html_url: str

    is_empty: bool

    name: str

    owner: str

    private: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .org import (
    OrgResource,
    AsyncOrgResource,
    OrgResourceWithRawResponse,
    AsyncOrgResourceWithRawResponse,
    OrgResourceWithStreamingResponse,
    AsyncOrgResourceWithStreamingResponse,
)
from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .client import (
    ClientResource,
    AsyncClientResource,
    ClientResourceWithRawResponse,
    AsyncClientResourceWithRawResponse,
    ClientResourceWithStreamingResponse,
    AsyncClientResourceWithStreamingResponse,
)

__all__ = [
    "ClientResource",
    "AsyncClientResource",
    "ClientResourceWithRawResponse",
    "AsyncClientResourceWithRawResponse",
    "ClientResourceWithStreamingResponse",
    "AsyncClientResourceWithStreamingResponse",
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
    "OrgResource",
    "AsyncOrgResource",
    "OrgResourceWithRawResponse",
    "AsyncOrgResourceWithRawResponse",
    "OrgResourceWithStreamingResponse",
    "AsyncOrgResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.github.webhook_receive_response import WebhookReceiveResponse

__all__ = ["WebhooksResource", "AsyncWebhooksResource"]


class WebhooksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WebhooksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return WebhooksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WebhooksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return WebhooksResourceWithStreamingResponse(self)

    def receive(
        self,
        *,
        x_github_delivery: str,
        x_github_event: str,
        x_hub_signature_256: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookReceiveResponse:
        """
        Receives and processes webhook events from GitHub.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {
            "X-GitHub-Delivery": x_github_delivery,
            "X-GitHub-Event": x_github_event,
            "X-Hub-Signature-256": x_hub_signature_256,
            **(extra_headers or {}),
        }
        return self._post(
            "/api/v1/github/webhooks/receiver",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookReceiveResponse,
        )


class AsyncWebhooksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWebhooksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncWebhooksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWebhooksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncWebhooksResourceWithStreamingResponse(self)

    async def receive(
        self,
        *,
        x_github_delivery: str,
        x_github_event: str,
        x_hub_signature_256: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookReceiveResponse:
        """
        Receives and processes webhook events from GitHub.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {
            "X-GitHub-Delivery": x_github_delivery,
            "X-GitHub-Event": x_github_event,
            "X-Hub-Signature-256": x_hub_signature_256,
            **(extra_headers or {}),
        }
        return await self._post(
            "/api/v1/github/webhooks/receiver",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookReceiveResponse,
        )


class WebhooksResourceWithRawResponse:
    def __init__(self, webhooks: WebhooksResource) -> None:
        self._webhooks = webhooks

        self.receive = to_raw_response_wrapper(
            webhooks.receive,
        )


class AsyncWebhooksResourceWithRawResponse:
    def __init__(self, webhooks: AsyncWebhooksResource) -> None:
        self._webhooks = webhooks

        self.receive = async_to_raw_response_wrapper(
            webhooks.receive,
        )


class WebhooksResourceWithStreamingResponse:
    def __init__(self, webhooks: WebhooksResource) -> None:
        self._webhooks = webhooks

        self.receive = to_streamed_response_wrapper(
            webhooks.receive,
        )


class AsyncWebhooksResourceWithStreamingResponse:
    def __init__(self, webhooks: AsyncWebhooksResource) -> None:
        self._webhooks = webhooks

        self.receive = async_to_streamed_response_wrapper(
            webhooks.receive,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ...._types import Body, Query, Headers, NotGiven, SequenceNotStr, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.tenants import batch_create_params, batch_delete_params, batch_update_params
from ....types.v1.create_tenant_param import CreateTenantParam
from ....types.v1.tenants.batch_create_response import BatchCreateResponse
from ....types.v1.tenants.batch_delete_response import BatchDeleteResponse
from ....types.v1.tenants.batch_update_response import BatchUpdateResponse

__all__ = ["BatchResource", "AsyncBatchResource"]


class BatchResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BatchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return BatchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BatchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return BatchResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        tenants: Iterable[CreateTenantParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchCreateResponse:
        """
        Create multiple tenants in a single request.

        Each tenant is processed independently. If one fails, others may succeed.
        Returns lists of succeeded and failed operations.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          tenants: List of tenants to create

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/tenants/batch",
            body=maybe_transform({"tenants": tenants}, batch_create_params.BatchCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchCreateResponse,
        )

    def update(
        self,
        *,
        tenants: Iterable[batch_update_params.Tenant],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchUpdateResponse:
        """
        Update multiple tenants in a single request.

        Each tenant is processed independently. If one fails, others may succeed.
        Returns lists of succeeded and failed operations.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          tenants: List of tenant updates

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._patch(
            "/api/v1/tenants/batch",
            body=maybe_transform({"tenants": tenants}, batch_update_params.BatchUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchUpdateResponse,
        )

    def delete(
        self,
        *,
        tenant_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchDeleteResponse:
        """
        Delete multiple tenants in a single request.

        Each tenant is processed independently. If one fails, others may succeed.
        Returns lists of succeeded and failed operations.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          tenant_ids: List of tenant IDs to delete

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._delete(
            "/api/v1/tenants/batch",
            body=maybe_transform({"tenant_ids": tenant_ids}, batch_delete_params.BatchDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchDeleteResponse,
        )


class AsyncBatchResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBatchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncBatchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBatchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncBatchResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        tenants: Iterable[CreateTenantParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchCreateResponse:
        """
        Create multiple tenants in a single request.

        Each tenant is processed independently. If one fails, others may succeed.
        Returns lists of succeeded and failed operations.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          tenants: List of tenants to create

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/tenants/batch",
            body=await async_maybe_transform({"tenants": tenants}, batch_create_params.BatchCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchCreateResponse,
        )

    async def update(
        self,
        *,
        tenants: Iterable[batch_update_params.Tenant],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchUpdateResponse:
        """
        Update multiple tenants in a single request.

        Each tenant is processed independently. If one fails, others may succeed.
        Returns lists of succeeded and failed operations.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          tenants: List of tenant updates

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._patch(
            "/api/v1/tenants/batch",
            body=await async_maybe_transform({"tenants": tenants}, batch_update_params.BatchUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchUpdateResponse,
        )

    async def delete(
        self,
        *,
        tenant_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchDeleteResponse:
        """
        Delete multiple tenants in a single request.

        Each tenant is processed independently. If one fails, others may succeed.
        Returns lists of succeeded and failed operations.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          tenant_ids: List of tenant IDs to delete

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._delete(
            "/api/v1/tenants/batch",
            body=await async_maybe_transform({"tenant_ids": tenant_ids}, batch_delete_params.BatchDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchDeleteResponse,
        )


class BatchResourceWithRawResponse:
    def __init__(self, batch: BatchResource) -> None:
        self._batch = batch

        self.create = to_raw_response_wrapper(
            batch.create,
        )
        self.update = to_raw_response_wrapper(
            batch.update,
        )
        self.delete = to_raw_response_wrapper(
            batch.delete,
        )


class AsyncBatchResourceWithRawResponse:
    def __init__(self, batch: AsyncBatchResource) -> None:
        self._batch = batch

        self.create = async_to_raw_response_wrapper(
            batch.create,
        )
        self.update = async_to_raw_response_wrapper(
            batch.update,
        )
        self.delete = async_to_raw_response_wrapper(
            batch.delete,
        )


class BatchResourceWithStreamingResponse:
    def __init__(self, batch: BatchResource) -> None:
        self._batch = batch

        self.create = to_streamed_response_wrapper(
            batch.create,
        )
        self.update = to_streamed_response_wrapper(
            batch.update,
        )
        self.delete = to_streamed_response_wrapper(
            batch.delete,
        )


class AsyncBatchResourceWithStreamingResponse:
    def __init__(self, batch: AsyncBatchResource) -> None:
        self._batch = batch

        self.create = async_to_streamed_response_wrapper(
            batch.create,
        )
        self.update = async_to_streamed_response_wrapper(
            batch.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            batch.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .batch import (
    BatchResource,
    AsyncBatchResource,
    BatchResourceWithRawResponse,
    AsyncBatchResourceWithRawResponse,
    BatchResourceWithStreamingResponse,
    AsyncBatchResourceWithStreamingResponse,
)
from .import_ import (
    ImportResource,
    AsyncImportResource,
    ImportResourceWithRawResponse,
    AsyncImportResourceWithRawResponse,
    ImportResourceWithStreamingResponse,
    AsyncImportResourceWithStreamingResponse,
)
from .tenants import (
    TenantsResource,
    AsyncTenantsResource,
    TenantsResourceWithRawResponse,
    AsyncTenantsResourceWithRawResponse,
    TenantsResourceWithStreamingResponse,
    AsyncTenantsResourceWithStreamingResponse,
)

__all__ = [
    "BatchResource",
    "AsyncBatchResource",
    "BatchResourceWithRawResponse",
    "AsyncBatchResourceWithRawResponse",
    "BatchResourceWithStreamingResponse",
    "AsyncBatchResourceWithStreamingResponse",
    "ImportResource",
    "AsyncImportResource",
    "ImportResourceWithRawResponse",
    "AsyncImportResourceWithRawResponse",
    "ImportResourceWithStreamingResponse",
    "AsyncImportResourceWithStreamingResponse",
    "TenantsResource",
    "AsyncTenantsResource",
    "TenantsResourceWithRawResponse",
    "AsyncTenantsResourceWithRawResponse",
    "TenantsResourceWithStreamingResponse",
    "AsyncTenantsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tenant import Tenant as Tenant
from .connection import Connection as Connection
from .client_user import ClientUser as ClientUser
from .group_detail import GroupDetail as GroupDetail
from .client_user_role import ClientUserRole as ClientUserRole
from .create_tenant_param import CreateTenantParam as CreateTenantParam
from .group_create_params import GroupCreateParams as GroupCreateParams
from .group_list_response import GroupListResponse as GroupListResponse
from .group_update_params import GroupUpdateParams as GroupUpdateParams
from .tenant_create_params import TenantCreateParams as TenantCreateParams
from .tenant_list_response import TenantListResponse as TenantListResponse
from .tenant_update_params import TenantUpdateParams as TenantUpdateParams
from .database_config_param import DatabaseConfigParam as DatabaseConfigParam
from .github_connect_params import GitHubConnectParams as GitHubConnectParams
from .github_callback_params import GitHubCallbackParams as GitHubCallbackParams
from .github_connect_response import GitHubConnectResponse as GitHubConnectResponse
from .connection_create_params import ConnectionCreateParams as ConnectionCreateParams
from .connection_list_response import ConnectionListResponse as ConnectionListResponse
from .connection_sync_response import ConnectionSyncResponse as ConnectionSyncResponse
from .connection_update_params import ConnectionUpdateParams as ConnectionUpdateParams
from .github_get_status_response import GitHubGetStatusResponse as GitHubGetStatusResponse
from .github_get_installation_link_response import (
    GitHubGetInstallationLinkResponse as GitHubGetInstallationLinkResponse,
)
from .connection_retrieve_credential_response import (
    ConnectionRetrieveCredentialResponse as ConnectionRetrieveCredentialResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .me import (
    MeResource,
    AsyncMeResource,
    MeResourceWithRawResponse,
    AsyncMeResourceWithRawResponse,
    MeResourceWithStreamingResponse,
    AsyncMeResourceWithStreamingResponse,
)
from .org.org import (
    OrgResource,
    AsyncOrgResource,
    OrgResourceWithRawResponse,
    AsyncOrgResourceWithRawResponse,
    OrgResourceWithStreamingResponse,
    AsyncOrgResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .github.github import (
    GitHubResource,
    AsyncGitHubResource,
    GitHubResourceWithRawResponse,
    AsyncGitHubResourceWithRawResponse,
    GitHubResourceWithStreamingResponse,
    AsyncGitHubResourceWithStreamingResponse,
)
from .groups.groups import (
    GroupsResource,
    AsyncGroupsResource,
    GroupsResourceWithRawResponse,
    AsyncGroupsResourceWithRawResponse,
    GroupsResourceWithStreamingResponse,
    AsyncGroupsResourceWithStreamingResponse,
)
from .tenants.tenants import (
    TenantsResource,
    AsyncTenantsResource,
    TenantsResourceWithRawResponse,
    AsyncTenantsResourceWithRawResponse,
    TenantsResourceWithStreamingResponse,
    AsyncTenantsResourceWithStreamingResponse,
)
from .connections.connections import (
    ConnectionsResource,
    AsyncConnectionsResource,
    ConnectionsResourceWithRawResponse,
    AsyncConnectionsResourceWithRawResponse,
    ConnectionsResourceWithStreamingResponse,
    AsyncConnectionsResourceWithStreamingResponse,
)

__all__ = ["V1Resource", "AsyncV1Resource"]


class V1Resource(SyncAPIResource):
    @cached_property
    def connections(self) -> ConnectionsResource:
        return ConnectionsResource(self._client)

    @cached_property
    def github(self) -> GitHubResource:
        return GitHubResource(self._client)

    @cached_property
    def groups(self) -> GroupsResource:
        return GroupsResource(self._client)

    @cached_property
    def me(self) -> MeResource:
        return MeResource(self._client)

    @cached_property
    def org(self) -> OrgResource:
        return OrgResource(self._client)

    @cached_property
    def tenants(self) -> TenantsResource:
        return TenantsResource(self._client)

    @cached_property
    def with_raw_response(self) -> V1ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return V1ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> V1ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return V1ResourceWithStreamingResponse(self)


class AsyncV1Resource(AsyncAPIResource):
    @cached_property
    def connections(self) -> AsyncConnectionsResource:
        return AsyncConnectionsResource(self._client)

    @cached_property
    def github(self) -> AsyncGitHubResource:
        return AsyncGitHubResource(self._client)

    @cached_property
    def groups(self) -> AsyncGroupsResource:
        return AsyncGroupsResource(self._client)

    @cached_property
    def me(self) -> AsyncMeResource:
        return AsyncMeResource(self._client)

    @cached_property
    def org(self) -> AsyncOrgResource:
        return AsyncOrgResource(self._client)

    @cached_property
    def tenants(self) -> AsyncTenantsResource:
        return AsyncTenantsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncV1ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncV1ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncV1ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncV1ResourceWithStreamingResponse(self)


class V1ResourceWithRawResponse:
    def __init__(self, v1: V1Resource) -> None:
        self._v1 = v1

    @cached_property
    def connections(self) -> ConnectionsResourceWithRawResponse:
        return ConnectionsResourceWithRawResponse(self._v1.connections)

    @cached_property
    def github(self) -> GitHubResourceWithRawResponse:
        return GitHubResourceWithRawResponse(self._v1.github)

    @cached_property
    def groups(self) -> GroupsResourceWithRawResponse:
        return GroupsResourceWithRawResponse(self._v1.groups)

    @cached_property
    def me(self) -> MeResourceWithRawResponse:
        return MeResourceWithRawResponse(self._v1.me)

    @cached_property
    def org(self) -> OrgResourceWithRawResponse:
        return OrgResourceWithRawResponse(self._v1.org)

    @cached_property
    def tenants(self) -> TenantsResourceWithRawResponse:
        return TenantsResourceWithRawResponse(self._v1.tenants)


class AsyncV1ResourceWithRawResponse:
    def __init__(self, v1: AsyncV1Resource) -> None:
        self._v1 = v1

    @cached_property
    def connections(self) -> AsyncConnectionsResourceWithRawResponse:
        return AsyncConnectionsResourceWithRawResponse(self._v1.connections)

    @cached_property
    def github(self) -> AsyncGitHubResourceWithRawResponse:
        return AsyncGitHubResourceWithRawResponse(self._v1.github)

    @cached_property
    def groups(self) -> AsyncGroupsResourceWithRawResponse:
        return AsyncGroupsResourceWithRawResponse(self._v1.groups)

    @cached_property
    def me(self) -> AsyncMeResourceWithRawResponse:
        return AsyncMeResourceWithRawResponse(self._v1.me)

    @cached_property
    def org(self) -> AsyncOrgResourceWithRawResponse:
        return AsyncOrgResourceWithRawResponse(self._v1.org)

    @cached_property
    def tenants(self) -> AsyncTenantsResourceWithRawResponse:
        return AsyncTenantsResourceWithRawResponse(self._v1.tenants)


class V1ResourceWithStreamingResponse:
    def __init__(self, v1: V1Resource) -> None:
        self._v1 = v1

    @cached_property
    def connections(self) -> ConnectionsResourceWithStreamingResponse:
        return ConnectionsResourceWithStreamingResponse(self._v1.connections)

    @cached_property
    def github(self) -> GitHubResourceWithStreamingResponse:
        return GitHubResourceWithStreamingResponse(self._v1.github)

    @cached_property
    def groups(self) -> GroupsResourceWithStreamingResponse:
        return GroupsResourceWithStreamingResponse(self._v1.groups)

    @cached_property
    def me(self) -> MeResourceWithStreamingResponse:
        return MeResourceWithStreamingResponse(self._v1.me)

    @cached_property
    def org(self) -> OrgResourceWithStreamingResponse:
        return OrgResourceWithStreamingResponse(self._v1.org)

    @cached_property
    def tenants(self) -> TenantsResourceWithStreamingResponse:
        return TenantsResourceWithStreamingResponse(self._v1.tenants)


class AsyncV1ResourceWithStreamingResponse:
    def __init__(self, v1: AsyncV1Resource) -> None:
        self._v1 = v1

    @cached_property
    def connections(self) -> AsyncConnectionsResourceWithStreamingResponse:
        return AsyncConnectionsResourceWithStreamingResponse(self._v1.connections)

    @cached_property
    def github(self) -> AsyncGitHubResourceWithStreamingResponse:
        return AsyncGitHubResourceWithStreamingResponse(self._v1.github)

    @cached_property
    def groups(self) -> AsyncGroupsResourceWithStreamingResponse:
        return AsyncGroupsResourceWithStreamingResponse(self._v1.groups)

    @cached_property
    def me(self) -> AsyncMeResourceWithStreamingResponse:
        return AsyncMeResourceWithStreamingResponse(self._v1.me)

    @cached_property
    def org(self) -> AsyncOrgResourceWithStreamingResponse:
        return AsyncOrgResourceWithStreamingResponse(self._v1.org)

    @cached_property
    def tenants(self) -> AsyncTenantsResourceWithStreamingResponse:
        return AsyncTenantsResourceWithStreamingResponse(self._v1.tenants)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel

__all__ = [
    "ConnectionRetrieveCredentialResponse",
    "PostgresCredentialResponse",
    "SnowflakeCredentialResponse",
    "SnowflakeCredentialResponseAuth",
    "SnowflakeCredentialResponseAuthSnowflakePasswordAuthResponse",
    "SnowflakeCredentialResponseAuthSnowflakePrivateKeyAuthResponse",
    "DatabricksCredentialResponse",
    "ClickHouseCredentialResponse",
    "MssqlCredentialResponse",
]


class PostgresCredentialResponse(BaseModel):
    """PostgreSQL credential response."""

    database: str
    """Database name"""

    host: str
    """Database host"""

    password: str
    """Decrypted password"""

    port: int
    """Database port"""

    username: str
    """Database username"""

    warehouse_type: Literal["postgresql"]
    """Warehouse type"""


class SnowflakeCredentialResponseAuthSnowflakePasswordAuthResponse(BaseModel):
    """Snowflake password authentication response."""

    auth_type: Literal["password"]
    """Authentication type"""

    password: str
    """Decrypted password"""


class SnowflakeCredentialResponseAuthSnowflakePrivateKeyAuthResponse(BaseModel):
    """Snowflake private key authentication response."""

    auth_type: Literal["private_key"]
    """Authentication type"""

    private_key: str
    """PEM-encoded private key"""

    private_key_passphrase: Optional[str] = None
    """Passphrase if key is encrypted"""


SnowflakeCredentialResponseAuth: TypeAlias = Annotated[
    Union[
        SnowflakeCredentialResponseAuthSnowflakePasswordAuthResponse,
        SnowflakeCredentialResponseAuthSnowflakePrivateKeyAuthResponse,
    ],
    PropertyInfo(discriminator="auth_type"),
]


class SnowflakeCredentialResponse(BaseModel):
    """Snowflake credential response."""

    account: str
    """Snowflake account identifier"""

    auth: SnowflakeCredentialResponseAuth
    """Authentication credentials"""

    role: str
    """Snowflake role"""

    username: str
    """Snowflake username"""

    warehouse: str
    """Compute warehouse name"""

    warehouse_type: Literal["snowflake"]
    """Warehouse type"""


class DatabricksCredentialResponse(BaseModel):
    """Databricks credential response."""

    access_token: str
    """Databricks personal access token"""

    http_path: str
    """SQL warehouse HTTP path"""

    server_hostname: str
    """Databricks server hostname"""

    warehouse_type: Literal["databricks"]
    """Warehouse type"""


class ClickHouseCredentialResponse(BaseModel):
    """ClickHouse credential response."""

    host: str
    """ClickHouse host"""

    password: str
    """Decrypted password"""

    port: int
    """ClickHouse port"""

    username: str
    """ClickHouse username"""

    warehouse_type: Literal["clickhouse"]
    """Warehouse type"""

    database: Optional[str] = None
    """Database name"""


class MssqlCredentialResponse(BaseModel):
    """Microsoft SQL Server credential response."""

    host: str
    """SQL Server host"""

    password: str
    """Decrypted password"""

    port: int
    """SQL Server port"""

    username: str
    """SQL Server username"""

    warehouse_type: Literal["mssql"]
    """Warehouse type"""

    database: Optional[str] = None
    """Database name"""


ConnectionRetrieveCredentialResponse: TypeAlias = Annotated[
    Union[
        PostgresCredentialResponse,
        SnowflakeCredentialResponse,
        DatabricksCredentialResponse,
        ClickHouseCredentialResponse,
        MssqlCredentialResponse,
    ],
    PropertyInfo(discriminator="warehouse_type"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel
from .tenancy_type import TenancyType

__all__ = ["Client"]


class Client(BaseModel):
    """Response schema for a client/organization."""

    id: str

    can_auto_join_by_domain: bool

    created_at: datetime

    members_must_have_matching_domain: bool

    name: str

    tenancy_type: TenancyType
    """Type of tenancy for a client."""

    updated_at: datetime

    domain: Optional[str] = None

    logo_url: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from .batch import (
    BatchResource,
    AsyncBatchResource,
    BatchResourceWithRawResponse,
    AsyncBatchResourceWithRawResponse,
    BatchResourceWithStreamingResponse,
    AsyncBatchResourceWithStreamingResponse,
)
from .import_ import (
    ImportResource,
    AsyncImportResource,
    ImportResourceWithRawResponse,
    AsyncImportResourceWithRawResponse,
    ImportResourceWithStreamingResponse,
    AsyncImportResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ....types.v1 import tenant_create_params, tenant_update_params
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.tenant import Tenant
from ....types.v1.tenant_list_response import TenantListResponse

__all__ = ["TenantsResource", "AsyncTenantsResource"]


class TenantsResource(SyncAPIResource):
    @cached_property
    def batch(self) -> BatchResource:
        return BatchResource(self._client)

    @cached_property
    def import_(self) -> ImportResource:
        return ImportResource(self._client)

    @cached_property
    def with_raw_response(self) -> TenantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return TenantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TenantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return TenantsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        tenant_key: str,
        connection_id: Optional[str] | Omit = omit,
        database_name: Optional[str] | Omit = omit,
        group_names: Optional[SequenceNotStr[str]] | Omit = omit,
        tenant_name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tenant:
        """
        Create a new tenant.

        Validates the client's tenancy type and creates the tenant accordingly:

        - ROW tenancy: Creates tenant with optional group associations
        - DATABASE tenancy: Requires database_name, creates tenant with database
          association
        - NONE tenancy: Returns error (tenant creation not allowed)

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          tenant_key: Unique tenant identifier within the client

          connection_id: Connection ID (required for DATABASE tenancy type)

          database_name: Database name (required for DATABASE tenancy type)

          group_names: List of group names to associate with the tenant

          tenant_name: Human-readable tenant name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/tenants",
            body=maybe_transform(
                {
                    "tenant_key": tenant_key,
                    "connection_id": connection_id,
                    "database_name": database_name,
                    "group_names": group_names,
                    "tenant_name": tenant_name,
                },
                tenant_create_params.TenantCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tenant,
        )

    def retrieve(
        self,
        tenant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tenant:
        """
        Get a single tenant by ID.

        Returns the tenant with associated groups and databases.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: TenantNotFoundError: If tenant doesn't exist (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._get(
            f"/api/v1/tenants/{tenant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tenant,
        )

    def update(
        self,
        tenant_id: str,
        *,
        connection_id: Optional[str] | Omit = omit,
        database_name: Optional[str] | Omit = omit,
        group_names: Optional[SequenceNotStr[str]] | Omit = omit,
        tenant_name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tenant:
        """
        Update a tenant.

        Updates tenant metadata, group associations, and/or database associations. Group
        associations use replace semantics:

        - Omit group_names to leave existing associations unchanged
        - Pass empty list to remove all group associations
        - Pass list of names to replace all associations

        Database associations (DATABASE tenancy only):

        - Both connection_id and database_name must be provided together
        - Replaces existing database association

        RLS: Filtered to current client (ClientRLSDB).

        Raises: TenantNotFoundError: If tenant doesn't exist (404)

        Args:
          connection_id: Connection ID for database association (required with database_name)

          database_name: Database name for association (required with connection_id)

          group_names: New list of group names (replaces all existing associations)

          tenant_name: New human-readable tenant name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._patch(
            f"/api/v1/tenants/{tenant_id}",
            body=maybe_transform(
                {
                    "connection_id": connection_id,
                    "database_name": database_name,
                    "group_names": group_names,
                    "tenant_name": tenant_name,
                },
                tenant_update_params.TenantUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tenant,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantListResponse:
        """
        List all tenants for the client.

        Returns tenants with their associated groups and databases.

        RLS: Filtered to current client (ClientRLSDB).
        """
        return self._get(
            "/api/v1/tenants",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantListResponse,
        )

    def delete(
        self,
        tenant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a tenant.

        Soft-deletes the tenant and its group/database associations.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: TenantNotFoundError: If tenant doesn't exist (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/api/v1/tenants/{tenant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncTenantsResource(AsyncAPIResource):
    @cached_property
    def batch(self) -> AsyncBatchResource:
        return AsyncBatchResource(self._client)

    @cached_property
    def import_(self) -> AsyncImportResource:
        return AsyncImportResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTenantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncTenantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTenantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncTenantsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        tenant_key: str,
        connection_id: Optional[str] | Omit = omit,
        database_name: Optional[str] | Omit = omit,
        group_names: Optional[SequenceNotStr[str]] | Omit = omit,
        tenant_name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tenant:
        """
        Create a new tenant.

        Validates the client's tenancy type and creates the tenant accordingly:

        - ROW tenancy: Creates tenant with optional group associations
        - DATABASE tenancy: Requires database_name, creates tenant with database
          association
        - NONE tenancy: Returns error (tenant creation not allowed)

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          tenant_key: Unique tenant identifier within the client

          connection_id: Connection ID (required for DATABASE tenancy type)

          database_name: Database name (required for DATABASE tenancy type)

          group_names: List of group names to associate with the tenant

          tenant_name: Human-readable tenant name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/tenants",
            body=await async_maybe_transform(
                {
                    "tenant_key": tenant_key,
                    "connection_id": connection_id,
                    "database_name": database_name,
                    "group_names": group_names,
                    "tenant_name": tenant_name,
                },
                tenant_create_params.TenantCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tenant,
        )

    async def retrieve(
        self,
        tenant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tenant:
        """
        Get a single tenant by ID.

        Returns the tenant with associated groups and databases.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: TenantNotFoundError: If tenant doesn't exist (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return await self._get(
            f"/api/v1/tenants/{tenant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tenant,
        )

    async def update(
        self,
        tenant_id: str,
        *,
        connection_id: Optional[str] | Omit = omit,
        database_name: Optional[str] | Omit = omit,
        group_names: Optional[SequenceNotStr[str]] | Omit = omit,
        tenant_name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tenant:
        """
        Update a tenant.

        Updates tenant metadata, group associations, and/or database associations. Group
        associations use replace semantics:

        - Omit group_names to leave existing associations unchanged
        - Pass empty list to remove all group associations
        - Pass list of names to replace all associations

        Database associations (DATABASE tenancy only):

        - Both connection_id and database_name must be provided together
        - Replaces existing database association

        RLS: Filtered to current client (ClientRLSDB).

        Raises: TenantNotFoundError: If tenant doesn't exist (404)

        Args:
          connection_id: Connection ID for database association (required with database_name)

          database_name: Database name for association (required with connection_id)

          group_names: New list of group names (replaces all existing associations)

          tenant_name: New human-readable tenant name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return await self._patch(
            f"/api/v1/tenants/{tenant_id}",
            body=await async_maybe_transform(
                {
                    "connection_id": connection_id,
                    "database_name": database_name,
                    "group_names": group_names,
                    "tenant_name": tenant_name,
                },
                tenant_update_params.TenantUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tenant,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantListResponse:
        """
        List all tenants for the client.

        Returns tenants with their associated groups and databases.

        RLS: Filtered to current client (ClientRLSDB).
        """
        return await self._get(
            "/api/v1/tenants",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantListResponse,
        )

    async def delete(
        self,
        tenant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a tenant.

        Soft-deletes the tenant and its group/database associations.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: TenantNotFoundError: If tenant doesn't exist (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/api/v1/tenants/{tenant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class TenantsResourceWithRawResponse:
    def __init__(self, tenants: TenantsResource) -> None:
        self._tenants = tenants

        self.create = to_raw_response_wrapper(
            tenants.create,
        )
        self.retrieve = to_raw_response_wrapper(
            tenants.retrieve,
        )
        self.update = to_raw_response_wrapper(
            tenants.update,
        )
        self.list = to_raw_response_wrapper(
            tenants.list,
        )
        self.delete = to_raw_response_wrapper(
            tenants.delete,
        )

    @cached_property
    def batch(self) -> BatchResourceWithRawResponse:
        return BatchResourceWithRawResponse(self._tenants.batch)

    @cached_property
    def import_(self) -> ImportResourceWithRawResponse:
        return ImportResourceWithRawResponse(self._tenants.import_)


class AsyncTenantsResourceWithRawResponse:
    def __init__(self, tenants: AsyncTenantsResource) -> None:
        self._tenants = tenants

        self.create = async_to_raw_response_wrapper(
            tenants.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            tenants.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            tenants.update,
        )
        self.list = async_to_raw_response_wrapper(
            tenants.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tenants.delete,
        )

    @cached_property
    def batch(self) -> AsyncBatchResourceWithRawResponse:
        return AsyncBatchResourceWithRawResponse(self._tenants.batch)

    @cached_property
    def import_(self) -> AsyncImportResourceWithRawResponse:
        return AsyncImportResourceWithRawResponse(self._tenants.import_)


class TenantsResourceWithStreamingResponse:
    def __init__(self, tenants: TenantsResource) -> None:
        self._tenants = tenants

        self.create = to_streamed_response_wrapper(
            tenants.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            tenants.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            tenants.update,
        )
        self.list = to_streamed_response_wrapper(
            tenants.list,
        )
        self.delete = to_streamed_response_wrapper(
            tenants.delete,
        )

    @cached_property
    def batch(self) -> BatchResourceWithStreamingResponse:
        return BatchResourceWithStreamingResponse(self._tenants.batch)

    @cached_property
    def import_(self) -> ImportResourceWithStreamingResponse:
        return ImportResourceWithStreamingResponse(self._tenants.import_)


class AsyncTenantsResourceWithStreamingResponse:
    def __init__(self, tenants: AsyncTenantsResource) -> None:
        self._tenants = tenants

        self.create = async_to_streamed_response_wrapper(
            tenants.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            tenants.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            tenants.update,
        )
        self.list = async_to_streamed_response_wrapper(
            tenants.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tenants.delete,
        )

    @cached_property
    def batch(self) -> AsyncBatchResourceWithStreamingResponse:
        return AsyncBatchResourceWithStreamingResponse(self._tenants.batch)

    @cached_property
    def import_(self) -> AsyncImportResourceWithStreamingResponse:
        return AsyncImportResourceWithStreamingResponse(self._tenants.import_)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .client import Client as Client
from .tenancy_type import TenancyType as TenancyType
from .user_create_params import UserCreateParams as UserCreateParams
from .user_list_response import UserListResponse as UserListResponse
from .user_update_params import UserUpdateParams as UserUpdateParams
from .client_update_params import ClientUpdateParams as ClientUpdateParams
from .user_delete_response import UserDeleteResponse as UserDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["Tenant", "Database", "Group"]


class Database(BaseModel):
    """Database information in tenant responses."""

    id: str
    """Database ID"""

    name: str
    """Database name"""


class Group(BaseModel):
    """Group information in tenant responses."""

    id: str
    """Group ID"""

    name: str
    """Group name"""


class Tenant(BaseModel):
    """Response model for a single tenant."""

    id: str
    """Tenant ID"""

    created_at: datetime
    """Creation timestamp"""

    databases: List[Database]
    """Associated databases"""

    groups: List[Group]
    """Associated groups"""

    name: Optional[str] = None
    """Human-readable tenant name"""

    tenant_key: str
    """Unique tenant identifier"""

    updated_at: datetime
    """Last update timestamp"""
